@@ -5,6 +5,7 @@
 #'
 #' @importFrom R6 R6Class
 #' @importFrom jsonlite toJSON
+#' @importFrom stats runif
 #' @export
 UnifiedLogger <- R6::R6Class("UnifiedLogger",
   public = list(
@@ -34,19 +35,15 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Initialize the unified logger
-    #' @param base_dir Base directory for logs (defaults to tempdir())
-    #' @param level Logging level: DEBUG, INFO, WARN, ERROR (default: "INFO")
-    #' @param max_size Maximum log file size in MB (default: 10)
-    #' @param max_files Maximum number of log files to keep (default: 5)
-    #' @param console_output Whether to output to console (default: TRUE)
-    #' @param json_format Whether to use JSON format (default: TRUE)
-    initialize = function(base_dir = NULL, level = "INFO", max_size = 10, 
+    #
+    #
+    #
+    #
+    #
+    #
+    initialize = function(base_dir = "logs", level = "INFO", max_size = 10, 
                          max_files = 5, console_output = TRUE, json_format = TRUE) {
-      if (is.null(base_dir)) {
-        self$log_dir <- file.path(tempdir(), "mLLMCelltype_logs")
-      } else {
-        self$log_dir <- base_dir
-      }
+      self$log_dir <- base_dir
       self$log_level <- toupper(level)
       self$max_log_size <- max_size
       self$max_log_files <- max_files
@@ -59,8 +56,8 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
         dir.create(self$log_dir, recursive = TRUE)
       }
       
-      # Generate session ID
-      self$session_id <- format(Sys.time(), "%Y%m%d_%H%M%S")
+      # Generate session ID with sub-second + random suffix to avoid collisions
+      self$session_id <- private$create_session_id()
       
       # Initialize performance stats
       self$performance_stats <- list(
@@ -82,24 +79,24 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log a debug message
-    #' @param message Log message
-    #' @param context Additional context (optional)
+    #
+    #
     debug = function(message, context = NULL) {
       private$log_message("DEBUG", message, context)
     },
     
     #' @description
     #' Log an info message
-    #' @param message Log message
-    #' @param context Additional context (optional)
+    #
+    #
     info = function(message, context = NULL) {
       private$log_message("INFO", message, context)
     },
     
     #' @description
     #' Log a warning message
-    #' @param message Log message
-    #' @param context Additional context (optional)
+    #
+    #
     warn = function(message, context = NULL) {
       self$performance_stats$warnings <- self$performance_stats$warnings + 1
       private$log_message("WARN", message, context)
@@ -107,8 +104,8 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log an error message
-    #' @param message Log message
-    #' @param context Additional context (optional)
+    #
+    #
     error = function(message, context = NULL) {
       self$performance_stats$errors <- self$performance_stats$errors + 1
       private$log_message("ERROR", message, context)
@@ -116,11 +113,11 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log API call performance
-    #' @param provider API provider name
-    #' @param model Model name
-    #' @param duration Duration in seconds
-    #' @param success Whether the call was successful
-    #' @param tokens Number of tokens used (optional)
+    #
+    #
+    #
+    #
+    #
     log_api_call = function(provider, model, duration, success = TRUE, tokens = NULL) {
       self$performance_stats$api_calls <- self$performance_stats$api_calls + 1
       
@@ -142,12 +139,12 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log complete API request and response for debugging and audit
-    #' @param provider API provider name
-    #' @param model Model name
-    #' @param prompt_content The complete prompt sent to the API
-    #' @param response_content The complete response received from the API
-    #' @param request_metadata Additional request metadata (optional)
-    #' @param response_metadata Additional response metadata (optional)
+    #
+    #
+    #
+    #
+    #
+    #
     log_api_request_response = function(provider, model, prompt_content, response_content, 
                                       request_metadata = NULL, response_metadata = NULL) {
       # Skip during R CMD check
@@ -162,8 +159,9 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
       }
       
       # Generate unique log file name for this API call
-      timestamp <- format(Sys.time(), "%Y%m%d_%H%M%S_%f")
-      api_call_id <- sprintf("%s_%s_%s", provider, gsub("[^A-Za-z0-9_-]", "_", model), timestamp)
+      timestamp <- format(Sys.time(), "%Y%m%d_%H%M%S")
+      call_seq <- self$performance_stats$api_calls + 1
+      api_call_id <- sprintf("%s_%s_%s_%03d", provider, gsub("[^A-Za-z0-9_-]", "_", model), timestamp, call_seq)
       
       # Create complete API log entry
       api_log_entry <- list(
@@ -222,9 +220,9 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log cache operations
-    #' @param operation Operation type: "hit", "miss", "store", "clear"
-    #' @param key Cache key
-    #' @param size Size of cached data (optional)
+    #
+    #
+    #
     log_cache_operation = function(operation, key, size = NULL) {
       if (operation == "hit") {
         self$performance_stats$cache_hits <- self$performance_stats$cache_hits + 1
@@ -232,12 +230,12 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
         self$performance_stats$cache_misses <- self$performance_stats$cache_misses + 1
       }
       
+      total_cache_ops <- self$performance_stats$cache_hits + self$performance_stats$cache_misses
       context <- list(
         operation = operation,
         key = key,
         size_bytes = size,
-        cache_hit_rate = round(self$performance_stats$cache_hits / 
-                              (self$performance_stats$cache_hits + self$performance_stats$cache_misses), 3)
+        cache_hit_rate = if (total_cache_ops > 0) round(self$performance_stats$cache_hits / total_cache_ops, 3) else 0
       )
       
       self$debug(sprintf("Cache %s: %s", operation, key), context)
@@ -245,9 +243,9 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log cluster annotation progress
-    #' @param cluster_id Cluster identifier
-    #' @param stage Current stage
-    #' @param progress Progress information
+    #
+    #
+    #
     log_cluster_progress = function(cluster_id, stage, progress = NULL) {
       context <- list(
         cluster_id = cluster_id,
@@ -261,9 +259,9 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Log detailed cluster discussion with complete model conversations
-    #' @param cluster_id Cluster identifier  
-    #' @param event_type Type of event (start, prediction, consensus, end)
-    #' @param data Event data
+    #
+    #
+    #
     log_discussion = function(cluster_id, event_type, data = NULL) {
       # Skip during R CMD check
       if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", ""))) {
@@ -294,6 +292,8 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
       
       # Also create detailed cluster log file for easy access
       cluster_log_file <- file.path(session_dir, sprintf("cluster_%s_discussion.md", cluster_id))
+      # Pure dialogue log: model outputs only (no timestamps/session metadata)
+      dialogue_log_file <- file.path(session_dir, sprintf("cluster_%s_dialogue.txt", cluster_id))
       
       if (event_type == "start") {
         # Create markdown log for better readability
@@ -303,6 +303,7 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
                           if(is.null(data$marker_genes)) "Unknown" else data$marker_genes, 
                           format(Sys.time(), "%Y-%m-%d %H:%M:%S"))
         cat(content, file = cluster_log_file)
+        cat("", file = dialogue_log_file)
         
       } else if (event_type == "prediction") {
         # Append model prediction in readable format
@@ -318,28 +319,98 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
                           if(is.null(data$round)) "1" else data$round, 
                           prediction_text)
         cat(content, file = cluster_log_file, append = TRUE)
+        dialogue_content <- sprintf("%s (Round %s):\n%s\n\n",
+                                   as.character(data$model),
+                                   if (is.null(data$round)) "1" else as.character(data$round),
+                                   prediction_text)
+        cat(dialogue_content, file = dialogue_log_file, append = TRUE)
         
       } else if (event_type == "consensus") {
         # Log consensus result
         content <- sprintf("## Consensus Result\n\n**Reached:** %s\n**Proportion:** %.2f\n**Entropy:** %.2f\n**Decision:** %s\n\n---\n\n",
-                          data$reached %||% FALSE,
-                          data$consensus_proportion %||% 0,
-                          data$entropy %||% 0,
-                          data$majority_prediction %||% "Unknown")
+                          if (!is.null(data$reached)) data$reached else FALSE,
+                          if (!is.null(data$consensus_proportion)) data$consensus_proportion else 0,
+                          if (!is.null(data$entropy)) data$entropy else 0,
+                          if (!is.null(data$majority_prediction)) data$majority_prediction else "Unknown")
         cat(content, file = cluster_log_file, append = TRUE)
         
       } else if (event_type == "end") {
         # Final summary
         content <- sprintf("## Discussion Complete\n\n**Final Result:** %s\n**Completed:** %s\n",
-                          data$final_result %||% "Unknown",
+                          if (!is.null(data$final_result)) data$final_result else "Unknown",
                           format(Sys.time(), "%Y-%m-%d %H:%M:%S"))
         cat(content, file = cluster_log_file, append = TRUE)
       }
     },
+
+    #' @description
+    #' Log model response with concise summary in main log and full text in file
+    #
+    #
+    #
+    log_model_response = function(provider, model, response, stage = "annotation", cluster_id = NULL) {
+      # Skip file writes during R CMD check
+      if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", ""))) {
+        response_text <- if (is.character(response)) paste(response, collapse = "\n") else as.character(response)
+        self$info("Model response received", list(
+          provider = provider,
+          model = model,
+          stage = stage,
+          cluster_id = cluster_id,
+          response_chars = nchar(response_text)
+        ))
+        return(invisible(NULL))
+      }
+
+      session_dir <- file.path(self$log_dir, self$session_id)
+      if (!dir.exists(session_dir)) {
+        dir.create(session_dir, recursive = TRUE)
+      }
+      response_dir <- file.path(session_dir, "model_responses")
+      if (!dir.exists(response_dir)) {
+        dir.create(response_dir, recursive = TRUE)
+      }
+
+      response_text <- if (is.character(response)) paste(response, collapse = "\n") else as.character(response)
+      timestamp <- format(Sys.time(), "%Y%m%d_%H%M%S")
+      model_safe <- gsub("[^A-Za-z0-9_-]", "_", model)
+      provider_safe <- gsub("[^A-Za-z0-9_-]", "_", provider)
+      stage_safe <- gsub("[^A-Za-z0-9_-]", "_", stage)
+      cluster_safe <- if (is.null(cluster_id)) "na" else gsub("[^A-Za-z0-9_-]", "_", as.character(cluster_id))
+      nonce <- sprintf("%06d", as.integer(stats::runif(1, 0, 999999)))
+      response_file <- file.path(
+        response_dir,
+        sprintf("%s_%s_%s_cluster-%s_%s.txt", provider_safe, model_safe, stage_safe, cluster_safe, paste0(timestamp, "_", nonce))
+      )
+
+      writeLines(response_text, response_file)
+
+      preview <- if (nchar(response_text) > 180) {
+        paste0(substr(response_text, 1, 180), "...")
+      } else {
+        response_text
+      }
+
+      context <- list(
+        provider = provider,
+        model = model,
+        stage = stage,
+        response_chars = nchar(response_text),
+        response_lines = length(strsplit(response_text, "\n", fixed = TRUE)[[1]]),
+        response_file = response_file,
+        response_preview = preview
+      )
+      if (!is.null(cluster_id)) {
+        context$cluster_id <- cluster_id
+      }
+
+      self$info("Model response received", context)
+      invisible(response_file)
+    },
     
     #' @description
     #' Get performance summary
-    #' @return List of performance statistics
+    #
     get_performance_summary = function() {
       session_duration <- as.numeric(difftime(Sys.time(), self$performance_stats$session_start, units = "secs"))
       
@@ -362,7 +433,7 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Clean up old log files
-    #' @param force Force cleanup even if within file limits
+    #
     cleanup_logs = function(force = FALSE) {
       log_files <- list.files(self$log_dir, pattern = "^mllm_.*\\.log$", full.names = TRUE)
       
@@ -400,7 +471,7 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
     
     #' @description
     #' Set logging level
-    #' @param level New logging level: DEBUG, INFO, WARN, ERROR
+    #
     set_level = function(level) {
       old_level <- self$log_level
       self$log_level <- toupper(level)
@@ -409,6 +480,13 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
   ),
   
   private = list(
+    create_session_id = function() {
+      ts <- format(Sys.time(), "%Y%m%d_%H%M%S")
+      subsec <- sprintf("%03d", as.integer((as.numeric(Sys.time()) %% 1) * 1000))
+      nonce <- sprintf("%06d", as.integer(stats::runif(1, 0, 999999)))
+      sprintf("%s%s_%d_%s", ts, subsec, Sys.getpid(), nonce)
+    },
+
     # Internal method to write log messages
     # @param level Log level
     # @param message Log message  
@@ -488,23 +566,72 @@ UnifiedLogger <- R6::R6Class("UnifiedLogger",
   )
 )
 
-# Create package environment for storing the logger instance
-.mllm_env <- new.env(parent = emptyenv())
+# Package-level environment for logger storage (avoids modifying .GlobalEnv)
+.mllm_pkg_env <- new.env(parent = emptyenv())
 
 #' Get the global logger instance
-#' @return UnifiedLogger instance
+#
 #' @export
 get_logger <- function() {
-  if (!exists(".mllm_logger", envir = .mllm_env) || is.null(.mllm_env$.mllm_logger)) {
-    .mllm_env$.mllm_logger <- UnifiedLogger$new()
+  if (is.null(.mllm_pkg_env$.mllm_logger)) {
+    .mllm_pkg_env$.mllm_logger <- UnifiedLogger$new()
   }
-  return(.mllm_env$.mllm_logger)
+  return(.mllm_pkg_env$.mllm_logger)
+}
+
+#' Reinitialize global logger with a specific directory
+#'
+#' Preserves the current logger configuration (level, size, retention, console/json)
+#' while changing the log directory for a new annotation session.
+#'
+#' @param log_dir Directory for log files
+#' @return Invisible logger object
+#' @keywords internal
+initialize_logger <- function(log_dir = "logs") {
+  current_logger <- if (!is.null(.mllm_pkg_env$.mllm_logger)) .mllm_pkg_env$.mllm_logger else NULL
+
+  level <- "INFO"
+  max_size <- 10
+  max_files <- 5
+  console_output <- TRUE
+  json_format <- TRUE
+
+  has_logger_fields <- function(x) {
+    is.environment(x) &&
+      !is.null(x$log_level) &&
+      !is.null(x$max_log_size) &&
+      !is.null(x$max_log_files) &&
+      !is.null(x$enable_console) &&
+      !is.null(x$enable_json)
+  }
+
+  if (!is.null(current_logger) && has_logger_fields(current_logger)) {
+    if (!is.null(current_logger$log_level)) level <- current_logger$log_level
+    if (!is.null(current_logger$max_log_size)) max_size <- current_logger$max_log_size
+    if (!is.null(current_logger$max_log_files)) max_files <- current_logger$max_log_files
+    if (!is.null(current_logger$enable_console)) console_output <- current_logger$enable_console
+    if (!is.null(current_logger$enable_json)) json_format <- current_logger$enable_json
+  }
+
+  .mllm_pkg_env$.mllm_logger <- UnifiedLogger$new(
+    base_dir = log_dir,
+    level = level,
+    max_size = max_size,
+    max_files = max_files,
+    console_output = console_output,
+    json_format = json_format
+  )
+
+  invisible(.mllm_pkg_env$.mllm_logger)
 }
 
 #' Set global logger configuration
-#' @param level Logging level
-#' @param console_output Whether to output to console
-#' @param json_format Whether to use JSON format
+#'
+#' @param level Logging level: "DEBUG", "INFO", "WARN", or "ERROR". Default: "INFO"
+#' @param console_output Whether to enable console output. Default: TRUE
+#' @param json_format Whether to use JSON format for log messages. Default: TRUE
+#'
+#' @return Invisible logger object
 #' @export
 configure_logger <- function(level = "INFO", console_output = TRUE, json_format = TRUE) {
   logger <- get_logger()
@@ -516,8 +643,11 @@ configure_logger <- function(level = "INFO", console_output = TRUE, json_format
 }
 
 #' Convenience functions for logging
-#' @param message Log message
-#' @param context Additional context (optional)
+#'
+#' @param message Log message string
+#' @param context Optional context information (list or character)
+#'
+#' @return Invisible NULL
 #' @name logging_functions
 NULL
 
@@ -543,4 +673,4 @@ log_warn <- function(message, context = NULL) {
 #' @export
 log_error <- function(message, context = NULL) {
   get_logger()$error(message, context)
-}
\ No newline at end of file
+}
